==> bitfront-library-6.0.0.tgz: package/src/app/base/component/base-list.component.ts <==
import {Router,
        ActivatedRoute}       from '@angular/router';
import {Subject,ReplaySubject}              from 'rxjs';

import {BaseComponent}        from './base.component';
import {Item}                 from '../../shared/data/item';
import {Error}                from '../../shared/data/error';
import {BaseCRUDServiceInterface} from '../shared/base-interface.service';
import {BaseFilter}           from '../shared/base.filter';
import {MessageService}       from '../../shared/service/message.service';
import {ToolbarService}       from '../../shared/service/toolbar.service';
import {TableViewService}     from '../../shared/service/tableview.service';
import {GeneralUtils}         from '../../shared/service/general-utils.service';
import {PageStatus}           from '../../shared/global.constants';
import {BitToolbarComponent}  from '../../shared/component/bit-toolbar.component';
import {BitMessage}           from '../../shared/data/bitmessage';

/**
 * Clase base para implementar listados de elementos con paginación. Provee el 95% de la funcionalidad necesaria.
 */
export abstract class BaseListComponent extends BaseComponent
{
    displayFilter: boolean = true; // indicará si se debe o no mostrar el componente hijo que representa el filtro    
    breadcrumb: any[] = [];  //contiene la miga de pan de nuestro componente
    items: Item[];       // los elementos a mostrar en la página tras la búsqueda
    totalItems: number;  // número total de elementos recuperados de la consulta
    //heredado! status: PageStatus;  // determina en qué estado está la página: PageStatus.Init, PageStatus.SEARCHING, PageStatus.FINISH_SEARCH, PageStatus.ERROR
    
    autoQuerySubject: ReplaySubject<BaseFilter> = new ReplaySubject<BaseFilter>(1); //es necesario que sea publica para que con AOT se pueda acceder a ella
    protected filter: BaseFilter;  // contiene el filtro de búsqueda. Puede ser null porque la página no lo necesite    
    protected customToolbarActions: any = {}; 
    protected messages: Map<PageStatus, string>;  //mensajes a indicar en el footer
    protected activatedRoute: ActivatedRoute;          // ruta activa en este momento
    protected router: Router;
    protected dataService: BaseCRUDServiceInterface;   // servicio para poder conectar con el servidor
    protected paramsDataService: any;
    protected messageService: MessageService;          // para mostrar mensajes por pantalla al usuario
    protected tableViewService: TableViewService;
    protected generalUtils: GeneralUtils;

   /** Constructor de la clase. */
   constructor(dataService: BaseCRUDServiceInterface, 
               messageService: MessageService, 
               activatedRoute: ActivatedRoute,
               router: Router,
               toolbarService?: ToolbarService,
               componentId?: string,
               generalUtils?: GeneralUtils,
               tableViewService?: TableViewService) {
      super(messageService, toolbarService, componentId);
      this.dataService = dataService;
      this.toolbarService = toolbarService;
      this.activatedRoute = activatedRoute;
      this.router = router;
      this.generalUtils = generalUtils;
      this.tableViewService = tableViewService;

      //inicializamos los datos
      this.resetData();
      //mensajes para el footer
      this.messages = new Map<PageStatus, string>();
      this.messages.set(PageStatus.Init, "Introdueixi els criteris de cerca i premi 'Cerca'");
      this.messages.set(PageStatus.Searching, "Cercant...");
      this.messages.set(PageStatus.FinishSearch, "No s'ha trobat cap dada amb les dades introduïdes");
   }

   /** Prepara el valor de los campos del filtro antes de realizar la consulta. */
   abstract prepareDataForSubmit(): void;

   /** Determina si se debe autoejecutar la búsqueda en función de si nos envían el parametro query=true en la URL */
   init(filter: BaseFilter): void{
      //¿ejecutamos el filtro?
      this.filter = filter;
      let query = this.activatedRoute.snapshot.params["query"];      
      if (query == 'true') {                
        console.log("autoquery en marcha");
        if (this.filter.pagina != BaseFilter.PAGE || this.filter.resultados != BaseFilter.RESULTADOS_PAGE){
           this.autoQuerySubject.next(this.filter); 
        }
        this.search();
      }
   }

   /** Resetea la página como si entráramos de nuevo en ella. */
   resetData(): void {
      super.resetData();
      this.items        = null;
      this.totalItems   = 0;
      this.status       = PageStatus.Init;
      this.displayFilter = true;
      this.toolbar =  {  //opciones comunies del toolbar para los listados
                        "filter": {enable:false, visible:true}, 
                        "print": {enable:false, visible:true}, 
                        "export": {enable:false, visible:true},
                        "new": {enable:false, visible:false}
                      };
      this.updateToolbar();                
   }

   /** Se invocará cuando el componente paginator hijo nos comunique que el usuario ha cambiado el número de elementos a mostrar por página */
   onNewPageSize(pageSize: number): void{
      this.filter.pagina     = 1;       // reseteamos a la página inicial
      this.filter.resultados = pageSize;
      this.search(false);
   }

   /** Se invocará cuando el componente paginator hijo nos comunique que el usuario ha cambiado el número de página a mostrar */
   onPageChange(page: number): void{
      this.filter.pagina = page;
      this.search(false);
   }

   /** Cuando algún componente hijo como el filtro nos comunique que se debe realizar la búsqueda */
   onSearch(filter: BaseFilter, count: boolean = true){
     //no sabemos cuantos elementos tendremos aun. Ponerlo a undefined nos permite saber si lanzamos un count o nos quedamos con el numero de elemntos retornados
     this.totalItems = undefined; 
     this.filter = filter;
     this.prepareDataForSubmit();
     this.autoQuerySubject.next(this.filter);
     this.search(count);
   }

   /** Cuando resetean el filtro de busqueda */
   onReset(filter: BaseFilter){
     this.filter = filter;
   }

   /** Cuando se pulsa un botón del toolbar */
   onToolbarButtonPressed(button: string){
     console.log("Button pressed:"+button);
     if (this.customToolbarActions[button]) {
       this.customToolbarActions[button]();
     } else if (button === BitToolbarComponent.FILTER_BUTTON){
       this.showFilter();       
     }else if (button === BitToolbarComponent.PRINT_BUTTON){
       this.handlePrint();
     }else if (button === BitToolbarComponent.EXPORT_BUTTON){
       this.handleExport();
     }else if (button === BitToolbarComponent.NEW_BUTTON){
       this.handleNew();
     } else {
       this.otherAction(button);
     }
   }

   handlePrint() {
       window.print();
   }

   handleExport() {
       if (this.tableViewService) {
         this.tableViewService.sendExportMessage();
       } else {
         alert("exportamos...proximamente");
       }
   }

   handleNew() {
      this.router.navigate(['form'], {relativeTo: this.activatedRoute});
   }
   
   /**cualquier otra accion que no sea la típica de guardar o eliminar */
   otherAction(button: string): void {
     console.warn(`otherAction ${button} pressed`);
   }

   initializeSearch(count: boolean) {
     // Inicializamos los datos básicos (solo si es necesario)
     if (count) {
       this.totalItems = 0;
     }
     this.status = PageStatus.Searching;
   }

   /** Marca el estado como "búsqueda finalizada" (PageStatus.FinishSearch). */
   finishSearch(): void {
      this.status = PageStatus.FinishSearch;
      //controlamos si tenemos que activar o desactivar los botones de toolbar print y export
      if (this.totalItems > 0){ //hay resulstados
        this.toolbar[BitToolbarComponent.PRINT_BUTTON].enable = true;
        this.toolbar[BitToolbarComponent.EXPORT_BUTTON].enable = true;
        this.hideFilter(); //en este método se envía el toolbar
      }else{
        this.toolbar[BitToolbarComponent.PRINT_BUTTON].enable = false;
        this.toolbar[BitToolbarComponent.EXPORT_BUTTON].enable = false;
        this.updateToolbar();
      }
   }

   /** Realiza la búsqueda filtrada de elementos. */
   search(count: boolean = true): void { 
      // Evitamos iniciar una nueva búsqueda cuando la anterior aún está en curso (p. ej., double click)
      if (this.status === PageStatus.Searching) { return; }
      //inicializamos la búsqueda
      this.initializeSearch(count); 
      // Consultamos los datos
      this.dataService.getAllByFilter(this.filter, this.paramsDataService)
          .subscribe(
                result => { 
                  this.items = result; 
                  if(count) {
                    this.count();
                  } else {
                    // si no teníamos antes un total, es un listado no paginado y el total es la pagina actual
                    this.totalItems = this.totalItems || this.items.length;
                    this.finishSearch();
                  }
                  if (!this.items || this.items.length==0) {
                    this.showFilter();
                  }
                }, //finishSearch si no estamos pendiente de la cuenta
                error => { this.status = PageStatus.Error; this.items = null; }
          );
   }

   /** Realiza la búsqueda filtrada de elementos. */
   count(): void {
      this.dataService.countAllByFilter(this.filter, this.paramsDataService)
          .subscribe(
            result => { 
              this.totalItems = result == null?0:result; //por alguna razón que desconozco el servidor envía un 0 pero el observable interpreta un null
              this.finishSearch() 
            }, 
            error => { this.status = PageStatus.Error; this.items = null; }
          );
   } 

   /** Muestra el filtro de búsqueda. */
   showFilter(): void { 
     this.displayFilter = true;
     this.toolbar[BitToolbarComponent.FILTER_BUTTON].enable = false; 
     this.updateToolbar();
   }

   /** Oculta el filtro de búsqueda. */
   hideFilter(): void { 
     this.displayFilter = false;
     this.toolbar[BitToolbarComponent.FILTER_BUTTON].enable = true; 
     this.updateToolbar();
   }

}

==> bitfront-library-6.3.0.tgz: package/src/app/base/component/base-list.component.ts <==
import {Router,
        ActivatedRoute}       from '@angular/router';
import {ReplaySubject}      from 'rxjs';

import {BaseComponent}        from './base.component';
import {Item}                 from '../../shared/data/item';
import {BaseCRUDServiceInterface} from '../shared/base-interface.service';
import {BaseFilter}           from '../shared/base.filter';
import {MessageService}       from '../../shared/service/message.service';
import {ToolbarService}       from '../../shared/service/toolbar.service';
import {TableViewService}     from '../../shared/service/tableview.service';
import {GeneralUtils}         from '../../shared/service/general-utils.service';
import {PageStatus}           from '../../shared/global.constants';
import {BitToolbarComponent}  from '../../shared/component/bit-toolbar.component';

/**
 * Clase base para implementar listados de elementos con paginación. Provee el 95% de la funcionalidad necesaria.
 */
export abstract class BaseListComponent extends BaseComponent
{
    displayFilter: boolean = true; // indicará si se debe o no mostrar el componente hijo que representa el filtro    
    breadcrumb: any[] = [];  //contiene la miga de pan de nuestro componente
    items: Item[];       // los elementos a mostrar en la página tras la búsqueda
    totalItems: number;  // número total de elementos recuperados de la consulta
    //heredado! status: PageStatus;  // determina en qué estado está la página: PageStatus.Init, PageStatus.SEARCHING, PageStatus.FINISH_SEARCH, PageStatus.ERROR
    
    autoQuerySubject: ReplaySubject<BaseFilter> = new ReplaySubject<BaseFilter>(1); //es necesario que sea publica para que con AOT se pueda acceder a ella
    protected filter: BaseFilter;  // contiene el filtro de búsqueda. Puede ser null porque la página no lo necesite    
    protected customToolbarActions: any = {}; 
    protected messages: Map<PageStatus, string>;  //mensajes a indicar en el footer
    protected activatedRoute: ActivatedRoute;          // ruta activa en este momento
    protected router: Router;
    protected dataService: BaseCRUDServiceInterface;   // servicio para poder conectar con el servidor
    protected paramsDataService: any;
    protected messageService: MessageService;          // para mostrar mensajes por pantalla al usuario
    protected tableViewService: TableViewService;
    protected generalUtils: GeneralUtils;

   /** Constructor de la clase. */
   constructor(dataService: BaseCRUDServiceInterface, 
               messageService: MessageService, 
               activatedRoute: ActivatedRoute,
               router: Router,
               toolbarService?: ToolbarService,
               componentId?: string,
               generalUtils?: GeneralUtils,
               tableViewService?: TableViewService) {
      super(messageService, toolbarService, componentId);
      this.dataService = dataService;
      this.toolbarService = toolbarService;
      this.activatedRoute = activatedRoute;
      this.router = router;
      this.generalUtils = generalUtils;
      this.tableViewService = tableViewService;

      //inicializamos los datos
      this.resetData();
      //mensajes para el footer
      this.messages = new Map<PageStatus, string>();
      this.messages.set(PageStatus.Init, "Introdueixi els criteris de cerca i premi 'Cerca'");
      this.messages.set(PageStatus.Searching, "Cercant...");
      this.messages.set(PageStatus.FinishSearch, "No s'ha trobat cap dada amb les dades introduïdes");
   }

   /** Prepara el valor de los campos del filtro antes de realizar la consulta. */
   abstract prepareDataForSubmit(): void;

   /** Determina si se debe autoejecutar la búsqueda en función de si nos envían el parametro query=true en la URL */
   init(filter: BaseFilter): void{
      //¿ejecutamos el filtro?
      this.filter = filter;
      let query = this.activatedRoute.snapshot.params["query"];      
      if (query == 'true') {                
        console.log("autoquery en marcha");
        if (this.filter.pagina != BaseFilter.PAGE || this.filter.resultados != BaseFilter.RESULTADOS_PAGE){
           this.autoQuerySubject.next(this.filter); 
        }
        this.search();
      }
   }

   /** Resetea la página como si entráramos de nuevo en ella. */
   resetData(): void {
      super.resetData();
      this.items        = null;
      this.totalItems   = 0;
      this.status       = PageStatus.Init;
      this.displayFilter = true;
      this.toolbar =  {  //opciones comunies del toolbar para los listados
                        "filter": {enable:false, visible:true}, 
                        "print": {enable:false, visible:true}, 
                        "export": {enable:false, visible:true},
                        "new": {enable:false, visible:false}
                      };
      this.updateToolbar();                
   }

   /** Se invocará cuando el componente paginator hijo nos comunique que el usuario ha cambiado el número de elementos a mostrar por página */
   onNewPageSize(pageSize: number): void{
      this.filter.pagina     = 1;       // reseteamos a la página inicial para evitar comportamienos extraños
      this.filter.resultados = pageSize;
      this.search(false);
   }

   /** Se invocará cuando el componente paginator hijo nos comunique que el usuario ha cambiado el número de página a mostrar */
   onPageChange(page: number): void{
      this.filter.pagina = page;
      this.search(false);
   }

   /** Cuando algún componente hijo como el filtro nos comunique que se debe realizar la búsqueda */
   onSearch(filter: BaseFilter, count: boolean = true){
     //no sabemos cuantos elementos tendremos aun. Ponerlo a undefined nos permite saber si lanzamos un count o nos quedamos con el numero de elemntos retornados
     this.totalItems = undefined; 
     this.filter = filter;
     this.prepareDataForSubmit();
     this.autoQuerySubject.next(this.filter); // comunicamos el nuevo filtro para que el footer se actualice
     this.search(count);
   }

   /** Cuando resetean el filtro de busqueda */
   onReset(filter: BaseFilter){
     this.filter = filter;
   }

   /** Cuando se pulsa un botón del toolbar */
   onToolbarButtonPressed(button: string){
     console.log("Button pressed:"+button);
     if (this.customToolbarActions[button]) {
       this.customToolbarActions[button]();
     } else if (button === BitToolbarComponent.FILTER_BUTTON){
       this.showFilter();       
     }else if (button === BitToolbarComponent.PRINT_BUTTON){
       this.handlePrint();
     }else if (button === BitToolbarComponent.EXPORT_BUTTON){
       this.handleExport();
     }else if (button === BitToolbarComponent.NEW_BUTTON){
       this.handleNew();
     } else {
       this.otherAction(button);
     }
   }

   handlePrint() {
       window.print();
   }

   handleExport() {
       if (this.tableViewService) {
         this.tableViewService.sendExportMessage();
       } else {
         alert("Exportació no implementada per aquest llistat");
       }
   }

   handleNew() {
      this.router.navigate(['form'], {relativeTo: this.activatedRoute});
   }
   
   /**cualquier otra accion que no sea la típica de guardar o eliminar */
   otherAction(button: string): void {
     console.warn(`otherAction ${button} pressed`);
   }

   initializeSearch(count: boolean) {
     // Inicializamos los datos básicos (solo si es necesario)
     if (count) {
       this.totalItems = 0;
     }
     this.status = PageStatus.Searching;
   }

   /** Marca el estado como "búsqueda finalizada" (PageStatus.FinishSearch). */
   finishSearch(): void {
      this.status = PageStatus.FinishSearch;
      //controlamos si tenemos que activar o desactivar los botones de toolbar print y export
      if (this.totalItems > 0){ //hay resulstados
        this.toolbar[BitToolbarComponent.PRINT_BUTTON].enable = true;
        this.toolbar[BitToolbarComponent.EXPORT_BUTTON].enable = true;
        this.hideFilter(); //en este método se envía el toolbar
      }else{
        this.toolbar[BitToolbarComponent.PRINT_BUTTON].enable = false;
        this.toolbar[BitToolbarComponent.EXPORT_BUTTON].enable = false;
        this.updateToolbar();
      }
   }

   /** Realiza la búsqueda filtrada de elementos. */
   search(count: boolean = true): void { 
      // Evitamos iniciar una nueva búsqueda cuando la anterior aún está en curso (p. ej., double click)
      if (this.status === PageStatus.Searching) { return; }
      //inicializamos la búsqueda
      this.initializeSearch(count); 
      // Consultamos los datos
      this.dataService.getAllByFilter(this.filter, this.paramsDataService)
          .subscribe(
                result => { 
                  this.items = result; 
                  if(count) {
                    this.count();
                  } else {
                    // si no teníamos antes un total, es un listado no paginado y el total es la pagina actual
                    this.totalItems = this.totalItems || this.items.length;
                    this.finishSearch();
                  }
                  if (!this.items || this.items.length==0) {
                    this.showFilter();
                  }
                }, //finishSearch si no estamos pendiente de la cuenta
                error => { this.status = PageStatus.Error; this.items = null; }
          );
   }

   /** Realiza la búsqueda filtrada de elementos. */
   count(): void {
      this.dataService.countAllByFilter(this.filter, this.paramsDataService)
          .subscribe(
            result => { 
              this.totalItems = result || 0; //por alguna razón que desconozco el servidor envía un 0 pero el observable interpreta un null
              this.finishSearch() 
            }, 
            error => { this.status = PageStatus.Error; this.items = null; }
          );
   } 

   /** Muestra el filtro de búsqueda. */
   showFilter(): void { 
     this.displayFilter = true;
     this.toolbar[BitToolbarComponent.FILTER_BUTTON].enable = false; 
     this.updateToolbar();
   }

   /** Oculta el filtro de búsqueda. */
   hideFilter(): void { 
     this.displayFilter = false;
     this.toolbar[BitToolbarComponent.FILTER_BUTTON].enable = true; 
     this.updateToolbar();
   }

}
